﻿//Ethan Alexander Shulman 2018

using UnityEngine;
using System.Collections;
using EthansVRKit;
using EthansProceduralGeometry;

public class CourseGeneration : MonoBehaviour {

    public static float teeOffHeight;
    private static float groundHeight, xFreq, zFreq, xScale, zScale, xOffset, zOffset, xFreq2, zFreq2, xScale2, zScale2, teeOffRadius, holeRadius, holeX, holeZ,
        fairwayRadius, fairwayOffset, fairwayScale, fairwayWarp;

    public Material mapBackground;
    public Transform teeOff, hole, ground;
    public BoxCollider[] bounds;
    public ParticleSystem clouds;
    public Light sunLight;
    public GameObject[] detailObjects;
    public Vector2[] detailObjectScales, detailObjectRanges;

    private ProceduralGeometry pg;

    void Start()
    {
        pg = GetComponent<ProceduralGeometry>();
        pg.geometryChunkFunction = geometryChunk;
        Generate();
    }

    public void Generate()
    {
        groundHeight = Random.Range(-10.0f, 10.0f);
        teeOffHeight = groundHeight+Random.Range(0.5f,3.0f);
        teeOffRadius = Random.Range(4.0f, 12.0f);
        xFreq = Random.Range(0.01f, 0.2f);
        zFreq = Random.Range(0.01f, 0.2f);
        xScale = Random.Range(0.0f, 1.5f);
        zScale = Random.Range(0.0f, 1.5f);
        float tscale = Random.Range(0.1f,1f);
        xScale *= tscale;
        zScale *= tscale;
        xOffset = Random.Range(-Mathf.PI, Mathf.PI);
        zOffset = Random.Range(-Mathf.PI, Mathf.PI);

        /*xFreq2 = Random.Range(0.4f, 2.0f);
        zFreq2 = Random.Range(0.4f, 2.0f);
        xScale2 = Random.Range(-0.3f, 0.3f);
        zScale2 = Random.Range(-0.3f, 0.3f);
        */
        fairwayRadius = Random.Range(40.0f, 120.0f);
        fairwayOffset = Random.Range(-100.0f, 100.0f);
        fairwayWarp = Random.Range(10.0f, 200.0f);
        fairwayScale = Random.Range(1.0f,20.0f)/fairwayWarp;

        holeRadius = Random.Range(6.0f, 15.0f);
        holeX = Random.Range(-24.0f, 24.0f);
        holeZ = Random.Range(20.0f, 54.0f);

        VRClient.Client.LerpOverlayColor(Color.black, Color.black, 0.0f, null);
        StartCoroutine(waitForGeneration());
    }

    private IEnumerator waitForGeneration()
    {
        pg.UpdateMesh();
        while (pg.IsGenerating()) yield return new WaitForSeconds(0.5f);
        onLoadOverlayDone();
    }

    private readonly Vector4[] materialColors = new Vector4[] {
        new Vector4(10f/255f,96f/255f,26f/255f,1f), new Vector4(123f/255f,179f/255f,98f/255f,1f),
        new Vector4(43f/255f,153f/255f,47f/255f,1f), new Vector4(190f/255f,235f/255f,151f/255f,1f),
        new Vector4(86f/255f,241f/255f,23f/255f,1f), new Vector4(214f/255f,252f/255f,182f/255f,1f)
    };
    private void onLoadOverlayDone()
    {
        float hheight;
        int ib;
        geometry(0.0f, 0.0f, 0.0f, out hheight, out ib);
        hheight = -hheight;
        teeOff.localPosition = new Vector3(2.0f, hheight + 0.1f, 0.0f);
        Game.game.transform.localPosition = new Vector3(0.0f, hheight, 0.0f);
        geometry(holeX, 0.0f, holeZ, out hheight, out ib);
        Vector3 hopos = new Vector3(holeX, -hheight + 0.04f, holeZ);
        hole.localPosition = hopos;
        hole.localRotation = Quaternion.LookRotation(sampleDistanceDerivative(hopos.x,hopos.y,hopos.z).normalized);
        ground.localPosition = new Vector3(0.0f, groundHeight - (xScale + zScale + 0.5f), 0.0f);

        int diffuseColorId = Shader.PropertyToID("_DiffuseColor"),
    specularColorId = Shader.PropertyToID("_SpecularColor"),
    mcId = 0;
        Vector4 roff = Random.ColorHSV();
        Material[] mats = pg.materials;
        for (int i = 0; i < mats.Length; i++)
        {
            Material mat = mats[i];
            mat.SetColor(diffuseColorId, materialColors[mcId++] + roff * Random.Range(-0.1f, 0.1f));
            mat.SetColor(specularColorId, materialColors[mcId++] + roff* Random.Range(-0.1f, 0.1f));
        }

        int dobi = Random.Range(0, detailObjects.Length);
        Vector2 dobr = detailObjectRanges[dobi];
        int dobc = Random.Range((int)dobr.x, (int)dobr.y);
        GameObject dobo = detailObjects[dobi];
        Vector2 dobs = detailObjectScales[dobi];
        Material dobm = dobo.GetComponent<MeshRenderer>().sharedMaterial;
        Color cbase = Random.ColorHSV(0f,0.4f,0f,1f,0.3f,1f);
        dobm.SetColor(diffuseColorId, cbase);
        dobm.SetColor(specularColorId, Color.LerpUnclamped(cbase,Vector4.one,Random.value));
        for (int i = 0; i < dobc; i++)
        {
            Vector3 pos;
            pos.x = Random.Range(-40.0f, 40.0f);
            pos.z = Random.Range(-10.0f, 80.0f);
            geometry(pos.x, 0.0f, pos.z, out hheight, out ib);
            if (ib == 0)
            {
                Transform nob = GameObject.Instantiate(dobo).transform;
                pos.y = -hheight;
                float scale = Random.Range(dobs.x, dobs.y);
                nob.localPosition = pos + Vector3.up * (scale * Random.Range(0.3f,0.49f));
                nob.localRotation = Quaternion.Euler(Random.Range(-20f,20f), Random.Range(0.0f, 360.0f), Random.Range(-20f,20f));
                nob.localScale = Vector3.one * scale;
            }
        }

        bool cloudyDay = Random.value < 0.1f;

        float sunHeight = Random.Range(0.0f,180.0f);
        Vector4 sunsetColor = Random.value > 0.5f ? Random.ColorHSV(0.0f, 0.2f, 0.0f, 1.0f, 0.7f, 1.0f) : Random.ColorHSV(0.8f, 1.0f, 0.0f, 1.0f, 0.7f, 1.0f);
        sunLight.color = Vector4.LerpUnclamped(sunLight.color - Random.ColorHSV() * 0.15f, sunsetColor, Mathf.Pow(Mathf.Abs(sunHeight - 90f) / 90f,2f));
        sunLight.intensity += Random.Range(0.0f, 1.0f);
        if (cloudyDay) sunLight.intensity -= 1.5f;
        sunLight.transform.localRotation = Quaternion.Euler(sunHeight, Random.Range(0.0f, 360.0f), 0.0f);

        bool showClouds = cloudyDay || (Random.value < 0.5f);
        if (showClouds)
        {
            //clouds
            ParticleSystem.EmissionModule emiss = clouds.emission;
            emiss.rate = new ParticleSystem.MinMaxCurve(Random.Range(0.1f, 3.0f) + (cloudyDay ? 10.0f : 0.0f));
            clouds.startSpeed = Random.Range(0.0f, 10.0f + (cloudyDay ? 40.0f : 0.0f));
            float rcrd = Random.Range(0.2f,1.0f);
            clouds.startColor = new Vector4(rcrd,rcrd,rcrd,Random.Range(0.5f,1.0f));
            clouds.transform.localRotation = Quaternion.Euler(new Vector3(0.0f, Random.Range(0.0f, 360.0f), 0.0f));
            clouds.gameObject.SetActive(true);
        }

        mapBackground.SetFloat("_SunSize", Random.Range(0.6f, 0.9f));
        mapBackground.SetFloat("_Atmosphere", Random.Range(0.0f, cloudyDay ? 0.05f : 0.15f));
        mapBackground.SetVector("_SkyColor", Random.ColorHSV(195f / 359f, 210f / 359f, 105f / 255f, 120f / 255f, 200f / 255f, 220f / 255f) * (cloudyDay ? Random.Range(0.5f, 0.7f) : 1.0f));
        mapBackground.SetVector("_SunsetColor", sunsetColor);

        //RenderSettings.skybox = mapBackground;
        pg.meshDestination.SetActive(true);
        teeOff.gameObject.SetActive(true);
        hole.gameObject.SetActive(true);

        VRClient.Client.LerpOverlayColor(Color.black, Color.clear, 1.0f, null);
        Game.game.CourseLoaded();
    }

    private static ProceduralGeometryFunction geometryChunk(float x, float y, float z)
    {
        return geometry;
    }
    private static void geometry(float x, float y, float z, out float dist, out int mat)
    {
        float dx = x < 0.0f ? -x : x,
            dz = z < 0.0f ? -z : z,
        tod = Mathf.Sqrt(dx * dx + dz * dz)-teeOffRadius;
        if (tod < 0.0f)
        {
            dist = y-teeOffHeight;
            mat = 2;
        }
        else
        {
            dist = y+Mathf.Sin(x * xFreq + xOffset) * xScale + Mathf.Cos(z * zFreq + zOffset) * zScale - groundHeight;

            dx = x - holeX;
            if (dx < 0.0f) dx = -dx;
            dz = z - holeZ;
            if (dz < 0.0f) dz = -dz;
            tod = Mathf.Sqrt(dx * dx + dz * dz) - holeRadius;
            if (tod < 0.0f)
            {
                mat = 2;
            }
            else
            {
                if (x*x+Mathf.Sin(z*fairwayScale+fairwayOffset)*fairwayWarp < fairwayRadius) mat = 1;
                else mat = 0;
            }
        }
    }

    private Vector3 sampleDistanceDerivative(float x, float y, float z)
    {
        float bdist, dist;
        int mat;
        Vector3 v;
        geometry(x, y, z, out bdist, out mat);
        geometry(x + 0.1f, y, z, out dist, out mat);
        v.x = dist - bdist;
        geometry(x, y + 0.1f, z, out dist, out mat);
        v.y = dist - bdist;
        geometry(x, y, z + 0.1f, out dist, out mat);
        v.z = dist - bdist;
        return v;
    }
}
