﻿//Ethan Alexander Shulman 2018

using UnityEngine;
using UnityEngine.UI;
using EthansVRKit;
using System.Collections;

public class Game : MonoBehaviour
{
    public static Game game;

    public Transform golfBall, hole, shotDirection, menuButtons, powerBarArrow, powerBar, golfClub, wind;
    public GameObject backButton, endMenu, scoreBeam, menu;
    public Text backButtonText, tipText, scoreText;
    public AudioSource ballHitSfx, windSfx;
    public Scrollbar shadowQuality, aaQuality, mouseSensitivity, fieldOfView;

    [HideInInspector]
    public int state = STATE_LOADING;

    private const int STATE_LOADING = 999,
        STATE_AIMING = 0, 
        STATE_FIRING = 1, 
        STATE_FOLLOWING = 2;

    private Rigidbody gBody;
    private VRClient client;
    private Transform ctransform, mtransform;
    private float shootTime;
    private bool menuButtonsOpen = false;
    private int shotCount = 0, preMenuState,
        shadowQualityValue, aaQualityValue, fieldOfViewValue;
    private float mouseSensitivityValue;

    private string oldTipText;
    private static bool isFirstHole = true;

    void Awake()
    {
        game = this;
        mtransform = transform;
        ctransform = Camera.main.transform;
        gBody = golfBall.GetComponent<Rigidbody>();
        StartCoroutine(secondTimer());
    }

    void Start()
    {
        client = VRClient.Client;

        if (!isFirstHole) tipText.gameObject.SetActive(false);

        if (PlayerPrefs.HasKey("sq"))
        {
            shadowQualityValue = PlayerPrefs.GetInt("sq");
            QualitySettings.SetQualityLevel(shadowQualityValue);
            aaQualityValue = PlayerPrefs.GetInt("aa");
            client.SetRenderScale(1.0f+aaQualityValue/2.7f);
            mouseSensitivityValue = PlayerPrefs.GetFloat("ms",3f);
            client.MouseSensitivity = mouseSensitivityValue;
            fieldOfViewValue = PlayerPrefs.GetInt("fv");
            Camera.main.fieldOfView = 60f + (fieldOfViewValue / 9f) * 30f;
        }
        else
        {
            shadowQualityValue = 1;
            aaQualityValue = 0;
            mouseSensitivityValue = 3f;
            fieldOfViewValue = 0;
        }
    }

    void Update()
    {
        if (state == STATE_AIMING)
        {
            Vector3 eua = ctransform.localEulerAngles;
            eua.x = 90.0f;
            eua.z = 0.0f;
            shotDirection.localRotation = Quaternion.Euler(eua);
            if (client.touching && client.touchingChanged && client.ViewedObject == null)
            {
                Vector3 cpos = ctransform.position;
                if (!golfBall.gameObject.activeSelf)
                {
                    golfBall.localPosition = cpos + new Vector3(0.5f, -1.9f, 0.3f);
                    golfBall.gameObject.SetActive(true);
                }
                powerBar.gameObject.SetActive(true);
                golfClub.position = golfBall.localPosition+new Vector3(0.0f,1.5f,0.0f)-shotDirection.up*0.2f;
                golfClub.gameObject.SetActive(true);

                oldTipText = tipText.text;
                tipText.text = "Click to shoot.";

                backButton.SetActive(true);
                state = STATE_FIRING;
            }
        }
        else if (state == STATE_FIRING)
        {
            shootTime = Time.timeSinceLevelLoad;
            float power = shootTime%1.0f;
            if (power > 0.5f) power = 1.0f - (power-0.5f)*2.0f;
            else power *= 2.0f;
            powerBarArrow.localPosition = new Vector3(0, (power-0.5f)*100.0f, 0);
            golfClub.localRotation = Quaternion.Euler(0.0f, shotDirection.localEulerAngles.y+90.0f, -10.0f+power * 120.0f);
            if (client.touching && client.touchingChanged && client.ViewedObject == null)
            {
                ballHitSfx.volume = 0.001f+Mathf.Pow(power,4.0f)*3.0f;
                ballHitSfx.pitch = Random.Range(0.8f, 1.2f);
                ballHitSfx.Play();

                menuButtonsOpen = false;
                backButton.SetActive(false);
                menuButtons.gameObject.SetActive(false);
                shotDirection.gameObject.SetActive(false);
                powerBar.gameObject.SetActive(false);
                golfClub.gameObject.SetActive(false);
                state = STATE_FOLLOWING;

                shotCount++;
                gBody.constraints = RigidbodyConstraints.None;
                Vector3 svec = shotDirection.up;
                svec.y = 0.6f;
                svec *= Mathf.Pow(power,1.3f)*800.0f+10.0f;
                gBody.AddForce(svec);
            }
        }
        else if (state == STATE_FOLLOWING)
        {
            float vmag = gBody.velocity.magnitude;
            Vector3 cpos = mtransform.localPosition,
                tpos = golfBall.localPosition;

            float tshot = Time.timeSinceLevelLoad - shootTime;
            mtransform.localPosition = cpos + (tpos - cpos)*Time.deltaTime*0.1f*vmag;
            if (tshot > 5.0f)
            {
                gBody.drag = tshot - 5.0f;
                if (vmag < 0.25f)
                {
                    gBody.constraints = RigidbodyConstraints.FreezeAll;
                    gBody.velocity = Vector3.zero;
                    gBody.angularVelocity = Vector3.zero;
                    gBody.drag = 0.0f;

                    cpos = hole.localPosition;
                    Vector3 dir = (cpos - tpos).normalized;
                    tpos += Quaternion.Euler(new Vector3(0.0f, 270.0f, 0.0f)) * dir;
                    mtransform.localPosition = tpos;
                    shotDirection.localPosition = golfBall.localPosition;
                    shotDirection.gameObject.SetActive(true);

                    tipText.text = oldTipText;

                    dir.y = 0.0f;
                    menuButtons.localRotation = Quaternion.LookRotation(dir);

                    menuButtons.gameObject.SetActive(true);
                    menuButtonsOpen = true;

                    state = STATE_AIMING;
                }
            }
        }
    }

    private IEnumerator secondTimer()
    {
        while (true)
        {
            if (menuButtonsOpen)
            {
                menuButtons.position = ctransform.position;
            }
            if (Random.value > 0.9f)
            {
                wind.localPosition = new Vector3(Random.Range(-10.0f, 10.0f), Random.Range(-10.0f, 10.0f), Random.Range(-10.0f, 10.0f));
                windSfx.pitch = Random.Range(0.2f, 2.5f);
                windSfx.volume = 0.01f+Mathf.Pow(Random.value,4.0f);
                windSfx.Play();
            }
            yield return new WaitForSeconds(1.0f);
        }
    }

    public void CourseLoaded()
    {
        EthansLighting.EthansLighting.ethansLighting.reflectionUpdateDelay = 3.0f;

        shotDirection.localPosition = ctransform.position + new Vector3(0.5f, -1.9f, 0.3f);
        shotDirection.gameObject.SetActive(true);
        menuButtons.gameObject.SetActive(true);
        menuButtonsOpen = true;

        state = STATE_AIMING;
    }

    public void OnClickBack()
    {
        if (state == STATE_LOADING) return;
        backButton.SetActive(false);
        shotDirection.gameObject.SetActive(true);
        if (shotCount == 0) golfBall.gameObject.SetActive(false);
        golfClub.gameObject.SetActive(false);
        powerBar.gameObject.SetActive(false);
        state = STATE_AIMING;
    }

    public void ScoreHole()
    {
        state = STATE_LOADING;
        menuButtonsOpen = true;
        menuButtons.gameObject.SetActive(true);
        tipText.gameObject.SetActive(false);
        golfBall.gameObject.SetActive(false);
        scoreBeam.SetActive(true);

        string t = "You scored ";
        if (shotCount == 1) t += "HOLE IN ONE!";
        else if (shotCount == 2) t += "Eagle!";
        else if (shotCount == 3) t += "Birdie!";
        else if (shotCount == 4) t += "Par.";
        else t += (shotCount-4) + " over par.";

        ulong holesPlayed = 0, totalScore = 0;
        if (PlayerPrefs.HasKey("c"))
        {
            string[] hstr = PlayerPrefs.GetString("c").Split(':');
            holesPlayed = ulong.Parse(hstr[0]);
            totalScore = ulong.Parse(hstr[1]);
        }

        holesPlayed++;
        totalScore += (ulong)shotCount;
        PlayerPrefs.SetString("c", holesPlayed + ":" + totalScore);
        PlayerPrefs.Save();

        t += System.Environment.NewLine + "Holes Played: "+holesPlayed+System.Environment.NewLine+"Average Score: ";
        totalScore /= holesPlayed;
        if (totalScore == 1) t += "HOLE IN ONE!";
        else if (totalScore == 2) t += "Eagle!";
        else if (totalScore == 3) t += "Birdie!";
        else if (totalScore == 4) t += "Par.";
        else t += (totalScore - 4) + " over par.";

        scoreText.text = t;

        isFirstHole = false;
        endMenu.SetActive(true);
    }

    public void NextCourse()
    {
        client.LerpOverlayColorAndLoadScene(Color.clear, Color.black, 1.0f, 1);
    }

    public void OpenMenu()
    {
        if (state == STATE_LOADING) return;
        shadowQuality.value = shadowQualityValue / (float)(shadowQuality.numberOfSteps-1);
        aaQuality.value = aaQualityValue / (float)(aaQuality.numberOfSteps-1);
        mouseSensitivity.value = (mouseSensitivityValue-1f) * 0.1f;
        fieldOfView.value = fieldOfViewValue / (float)(fieldOfView.numberOfSteps - 1);
        preMenuState = state;
        state = STATE_LOADING;
        tipText.gameObject.SetActive(false);
        menu.gameObject.SetActive(true);
    }

    public void CloseMenu()
    {
        menu.gameObject.SetActive(false);
        if (isFirstHole) tipText.gameObject.SetActive(true);
        state = preMenuState;
    }


    public void OnShadowQualityChange()
    {
        int sq = (int)((shadowQuality.value-0.001f) * shadowQuality.numberOfSteps);
        shadowQualityValue = sq;
        PlayerPrefs.SetInt("sq", sq);
        QualitySettings.SetQualityLevel(sq);
    }
    public void OnAntiAliasingChange()
    {
        int aa = (int)((aaQuality.value-0.001f) * aaQuality.numberOfSteps);
        aaQualityValue = aa;
        PlayerPrefs.SetInt("aa", aa);
        client.SetRenderScale(1.0f + aa / 2.7f);
    }
    public void OnMouseSensitivityChange()
    {
        float ms = 1.0f+mouseSensitivity.value*10.0f;
        mouseSensitivityValue = ms;
        PlayerPrefs.SetFloat("ms", ms);
        client.MouseSensitivity = ms;
    }
    public void OnFieldOfViewChange()
    {
        int fovl = (int)((fieldOfView.value-0.001f)*fieldOfView.numberOfSteps);
        fieldOfViewValue = fovl;
        PlayerPrefs.SetInt("fv", fovl);
        Camera.main.fieldOfView = 60f + (fovl/9f)*30f;
    }

    public void OpenCredits()
    {
        Application.OpenURL("https://xaloez.com/");
    }
    public void OpenForum()
    {
        Application.OpenURL("https://xaloez.com/forum/index.php?board=4.0");
    }
}
